@@ -425,7 +425,7 @@ float sap2_neighborhood_match (Constraint_list *CL, int s1, int s2, int r1, int
 /*                                                                                           */
 /*********************************************************************************************/
 Alignment * analyse_pdb ( Alignment *A, Alignment *ST, char *filename);
-Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *filename);
+Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *filename, int gapped);
 float **** analyse_pdb_residues ( Alignment *A, Constraint_list *CL, Pdb_param *pdb_param);
 
 float square_atom ( Atom *X);
@@ -516,6 +516,7 @@ int slow_coffee_evaluate_sub_aln         ( Alignment *IN,int *ns, int **ls, Cons
 Alignment * slow_coffee_evaluate_output          ( Alignment *IN,Constraint_list *CL);
 Alignment * non_extended_t_coffee_evaluate_output( Alignment *IN,Constraint_list *CL);
 Alignment * heuristic_coffee_evaluate_output     ( Alignment *IN,Constraint_list *CL);
+Alignment *coffee_seq_evaluate_output ( Alignment *IN, Constraint_list *CL);
 /*Old Function: To deprecate*/
 Alignment * coffee_evaluate_output ( Alignment *IN,Constraint_list *CL);
 
@@ -534,6 +535,7 @@ int muscle_profile_profile     (int *prf1, int *prf2, Constraint_list *CL);
 /*         FUNCTIONS FOR GETING THE COST : (Sequences) ->evaluate_residue_pair               */
 /*                                                                                           */
 /*********************************************************************************************/
+int initialize_scoring_scheme (Constraint_list *CL);
 int evaluate_blast_profile_score (Constraint_list *CL, int s1, int r1, int s2, int r2);
 int evaluate_aln_profile_score   (Constraint_list *CL, int s1, int r1, int s2, int r2);
 
@@ -558,6 +560,7 @@ int residue_pair_extended_list4rna                  ( Constraint_list *CL, Const
 
 int residue_pair_extended_list_raw                     ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int residue_pair_extended_list_pc                     ( Constraint_list *CL, int s1, int r1, int s2, int r2);
+int residue_pair_extended_list_4gp                     ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int residue_pair_extended_list                     ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 
 
@@ -812,7 +815,7 @@ Sequence *pavie_seq2noisy_seq (Sequence *S, int val,char *subst);
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Tue Oct 27 10:12:26 WEST 2009. */
+/*Tue Mar 30 19:55:28 WEDT 2010. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
