@@ -82,7 +82,7 @@ int apdb ( int argc, char *argv[])
 	int pdb_min_sim;
 	int pdb_max_sim;
 	int pdb_min_cov;
-	
+	int gapped;
 
 	
 	char *prot_blast_server;
@@ -807,6 +807,24 @@ declare_name (prot_db);
 			    /*Min_value*/ "any"         ,\
 			    /*Max Value*/ "any"          \
 		   );
+ 
+ get_cl_param(							\
+			    /*argc*/      argc          ,\
+			    /*argv*/      argv          ,\
+			    /*output*/    &le           ,\
+			    /*Name*/      "-gapped"    ,\
+			    /*Flag*/      &garbage      ,\
+			    /*TYPE*/      "D"         ,\
+			    /*OPTIONAL?*/ OPTIONAL      ,\
+			    /*MAX Nval*/  1             ,\
+			    /*DOC*/       "ND"          ,\
+ 			    /*Parameter*/&gapped       ,\
+	 		    /*Def 1*/    "0"      ,\
+			    /*Def 2*/    "1"      ,\
+			    /*Min_value*/ "any"         ,\
+			    /*Max Value*/ "any"          \
+		   );
+
  // set the correct mode:
  if ( strm (argv[0], "trmsd"))sprintf (mode, "trmsd");
    
@@ -835,7 +853,7 @@ declare_name (prot_db);
 		
 		pdb_param->md_threshold=md_threshold;
 		pdb_param->maximum_distance=maximum_distance;
-		
+	
 		if ( n_excluded_nb>0)
 		  pdb_param->n_excluded_nb=n_excluded_nb;
 		else if ( n_excluded_nb==-1)
@@ -936,7 +954,7 @@ declare_name (prot_db);
 		  }
 		else if ( strm (mode, "msa2tree") || strm (mode, "trmsd"))
 		  {
-		    EA=msa2struc_dist ( A, EA,F->name);
+		    EA=msa2struc_dist ( A, EA,F->name, gapped);
 		  }
 		le=display_output_filename ( le, "APDB_RESULT", "APDB_RESULT_FORMAT_01", apdb_outfile, CHECK);
 		
@@ -2115,10 +2133,16 @@ float **** analyse_pdb_residues ( Alignment *A, Constraint_list *CL, Pdb_param *
 	 free_int (pos, -1);
 	 return distances;
      }
+int pair_res_suitable4trmsd    (int s1,int col1, int col2, Alignment *A, int **pos, Pdb_param *PP, Constraint_list *CL,int *s);
+int aln_column_contains_gap (Alignment *A, int c);
+float aln2ncol4trmsd(Alignment *A, int **pos, Constraint_list *CL, int **lc);
+int pair_columns_suitable4trmsd(int col1, int col2, Alignment *A, int **pos, Pdb_param *PP, Constraint_list *CL,int *s);
+int column_is_suitable4trmsd(int col1,Alignment *A, int **pos, Pdb_param *PP, Constraint_list *CL,int *s);
 
 
 
-Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *results)
+NT_node trmsdmat2tree (float **dm, int **count,Alignment *A);
+Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *results, int gapped)
      {
 
        int **pos, c;
@@ -2126,9 +2150,9 @@ Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *results)
 	 int s1, s2, rs1, rs2;
 	 int col1, col2;
 	 float ****distances;
-	 float **dm;
-	 int **count;
-	 int   **dm_int;
+	 float **dm,**tdm;
+	 int **count,**tcount;
+	
 	 float min, max;
 	 
 	      /*Distances[Nseq][len_aln][4]
@@ -2140,7 +2164,7 @@ Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *results)
 	      */
 	 Pdb_param *pdb_param;
 	 Constraint_list *CL;
-	 int a, b, ncol;
+	 int a, b, ncol, npos,n;
 	 float d1, d2,delta;
 	 int wd1, wd2;
 	 int in_bubble=0;
@@ -2155,26 +2179,33 @@ Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *results)
 	 NT_node *T0,*T1,*T2,*PT, *POS;
 	 NT_node BT0, BT10,BT50, BT100,RBT;
 	 char **pair_pos_list;
-	 
+	 int min_ncol4trmsd=4;
 	 int ntree=0, ntree2;
 	 
 	 Alignment *B;
 	 char *pos_list;
 	 char *tot_pos_list;
+	 
 	 char *struc_tree10;
 	 char *struc_tree100;
 	 char *struc_tree50;
 	 char *struc_tree0;
+	 char *consense_file;
 	 
 	 char *color_struc_tree;
 	 int **score;
 	 int proceed=1;
-	 
+	 int **lc;
 
+	 lc=declare_int (A->nseq, 2);
+	 for (a=0; a<A->nseq; a++)lc[a][0]=a;
 	 
 	 declare_name(tot_pos_list);
 	 sprintf ( tot_pos_list, "%s.tot_pos_list", results);
-	 
+
+	 declare_name(consense_file);
+	 sprintf (consense_file, "%s.consense_output", results);
+
 	 declare_name(pos_list);
 	 sprintf ( pos_list, "%s.pos_list", results);
 
@@ -2211,6 +2242,8 @@ Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *results)
 		 proceed=0;
 	       }
 	   }
+
+	
 	 if (!proceed)
 	    printf_exit (EXIT_FAILURE, stderr, "\n\n---- ERROR: All provided sequences must have a valid PDB identifier [FATAL:tRMSD-%s]\n\n", PROGRAM);
 	 
@@ -2223,113 +2256,104 @@ Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *results)
 	     if (CL->T[rs1] &&  !(CL->T[rs1])->ca_dist)(CL->T[rs1])->ca_dist=measure_ca_distances(CL->T[rs1]);
 	   }	 
 	 pos=aln2pos_simple (A, A->nseq);
-	 dm=declare_float (A->nseq, A->nseq);
-	 dm_int=declare_int (A->nseq, A->nseq);
-	 count=declare_int (A->nseq, A->nseq);
-	 PP->maximum_distance=10000;
 	 
+	 dm=declare_float (A->nseq, A->nseq);
+	 count=declare_int (A->nseq, A->nseq);
+	 tdm=declare_float (A->nseq, A->nseq);
+	 tcount=declare_int (A->nseq, A->nseq);
+	 
+	 PP->maximum_distance=1000;
+	 sprintf ( PP->local_mode, "sphere");
+	 
+	 
+	 while ((npos=aln2ncol4trmsd(A,pos,CL,lc))<min_ncol4trmsd && A->nseq>1)
+	   {
+	     
+	     sort_int_inv (lc,2, 1, 0,A->nseq-1);
+	     fprintf ( stdout, "\nRemove Sequence [%s] that contains %d un-suitable positions\n", A->name[lc[0][0]], lc[0][1]);
+	     A=remove_seq_from_aln (A, A->name[lc[0][0]]);
+	     ungap_aln (A);
+	     pos=aln2pos_simple (A, A->nseq);
+	   }
+	 if (!A->nseq)
+	   {
+	     fprintf ( stderr, "\nERROR: No suitable pair of column supporting a tree [FATAL]\n");
+	     exit (EXIT_SUCCESS);
+	   }
+	 else
+	    fprintf ( stdout, "\n---- Number of usable positions: %d [%.2f %%]\n", npos, ((float)npos*100)/(float)A->len_aln);
+
+	 print_aln (A);
+	 HERE ("Gapped=%d", gapped);
 	 tl=vfopen (tot_pos_list, "w");
 	 for (ncol=0,ntree=0, col1=0; col1< A->len_aln; col1++)
 	   {
-	     int tree, cont;
-	     output_completion (stderr, col1, A->len_aln,1, "Sample Columns");
-	     for (cont=1,s1=0; s1<A->nseq; s1++)if ( is_gap (A->seq_al[s1][col1]))cont=0;//Stop if gap in column 1
-	     
-	     if ( cont==0)continue;
-	     
-	     for (s1=0; s1<A->nseq; s1++)for ( s2=0; s2<A->nseq; s2++){count[s1][s2]=0;dm[s1][s2]=0;}
-	     
-	     for ( ntree2=0,col2=0; col2<A->len_aln; col2++)
+	     int w,tree, cont;
+	     //output_completion (stderr, col1, A->len_aln,1, "Sample Columns");
+	     if (!gapped && aln_column_contains_gap (A, col1))continue;
+	     for ( cont=1,ntree2=0,col2=0; col2<A->len_aln; col2++)
 	       {
 		 for (s1=0; s1< A->nseq-1; s1++)
 		   {
+		     rs1=A->order[s1][0];
+		     if (!pair_res_suitable4trmsd (s1,col1, col2, A, pos, PP, CL, &w))continue;
 		     for ( s2=s1+1; s2<A->nseq; s2++)
 		       {
-			 rs1=A->order[s1][0];
+			 if (!pair_res_suitable4trmsd (s2,col1, col2, A, pos, PP, CL, &w))continue;
+			 
 			 rs2=A->order[s2][0];
-			 cont=1;
-			 
-			 if ( s1==s2){dm[s1][s2]=0;continue;}
-			 else if (!(CL->T[rs1]) || !(CL->T[rs2])){cont=0;}
-			 else if ( islower (A->seq_al[s1][col1]) || islower ( A->seq_al[s2][col1])){cont=0;}
-			 else if ( pos[s1][col1]<=0 || pos[s2][col1]<=0 ){cont=0;}
-			 if (pos[s1][col2]<=0 || pos[s2][col2]<=0 ){cont=0;}//stop if Gap in Column 2
-			 else if ( FABS((pos[s1][col2]-pos[s1][col1]))<=PP->n_excluded_nb){cont=0;}
-			 else if ( FABS((pos[s2][col2]-pos[s2][col1]))<=PP->n_excluded_nb){cont=0;}
-			 else if ( islower (A->seq_al[s1][col2]) || islower ( A->seq_al[s2][col2])){cont=0;}
-			 if ( cont==0){continue;}
-			  
-			 
 			 real_res1_col1=pos[s1][col1]-1;
 			 real_res1_col2=pos[s1][col2]-1;
-			 
 			 real_res2_col1=pos[s2][col1]-1;
 			 real_res2_col2=pos[s2][col2]-1;
 			 
 			 d1=(CL->T[rs1])->ca_dist[real_res1_col1][real_res1_col2];
 			 d2=(CL->T[rs2])->ca_dist[real_res2_col1][real_res2_col2];
-		
-			 if ( d1==UNDEFINED || d2 == UNDEFINED) continue;
 			 
-			 if ( strm ( PP->local_mode, "sphere"))
-			   {
-			     in_bubble= (d1<PP->maximum_distance && d2<PP->maximum_distance)?1:0;		   ;		   
-			   }
-			 else if ( strm ( PP->local_mode, "window"))
-			   {
-			     wd1=FABS((pos[s1][col2]-pos[s1][col1]));
-			     wd2=FABS((pos[s2][col2]-pos[s2][col1]));
-			     in_bubble= (wd1<PP->maximum_distance && wd2<PP->maximum_distance)?1:0;		   ;		   
-			   } 
-			 if (in_bubble)
-			   {
-			     delta=FABS((d1-d2));
-			     //delta=delta*delta;
-			     dm[s1][s2]=dm[s2][s1]+=delta;
-			     count[s1][s2]++;
-			     count[s2][s1]++;
-			   }
+			 delta=FABS((d1-d2));
+			 dm[s1][s2]=dm[s2][s1]+=delta;
+			 tdm[s1][s2]=tdm[s2][s1]+=delta;
+			 tcount[s1][s2]++;
+			 tcount[s2][s1]++;
+			 
+			 count[s1][s2]++;
+			 count[s2][s1]++;
 		       }
 		   }
 	       }
-	   
 	     
-	     min=max=-1;
-	     for (tree=1,s1=0; s1<A->nseq-1; s1++)
-	       for (s2=s1+1; s2<A->nseq; s2++)
-		 {
-		   if ( count [s1][s2])dm[s1][s2]=dm[s2][s1]=dm[s1][s2]/(float)count[s1][s2];
-		   else 
-		     {
-		       tree=0;
-		     }
-		   if (s1==0 && s2==1)min=max=dm[s1][s2];
-		   min=MIN(dm[s1][s2], min);
-		   max=MAX(dm[s1][s2], max);
-		 }
-	     if (!tree || min==-1)continue;
-	     for (s1=0; s1<A->nseq-1; s1++)
-	       for (s2=s1+1; s2<A->nseq; s2++)
-		 {
-		   dm_int[s1][s2]=dm_int[s2][s1]=((dm[s1][s2])/(max))*100;
-		 }
-	     POS[col1]=T1[ntree]=compute_std_tree_2 ( A, dm_int, "_TMODE_upgma");
-	     fprintf (tl, "\n>Tree_%d Column\n", col1+1);
-	     print_tree (T1[ntree], "newick", tl);
-	     ntree++;
+	     
+	     
+	     if ((POS[col1]=trmsdmat2tree (dm, count, A)))
+	       {
+		 T1[ntree]=POS[col1];
+		 fprintf (tl, "\n>Tree_%d Column\n", col1+1);
+		 print_tree (T1[ntree], "newick", tl);
+		 ntree++;
+	       }
 	   }
-	 
 	 vfclose (tl);
-	 if (!ntree)
-	   {
-	      fprintf ( stderr, "\nERROR: No suitable pair of column supporting a tree [FATAL]\n", (A->name[a]));
-	      exit (EXIT_SUCCESS);
-	   }
+	     
+	 if (!ntree){fprintf ( stderr, "\nERROR: No suitable pair of column supporting a tree [FATAL]\n"); exit (EXIT_SUCCESS);}
 	 
 	 score=treelist2avg_treecmp (T1, NULL);
-	 
 	 display_output_filename( stdout,"TreeList","newick",tot_pos_list, CHECK);
 	 
+	 if (treelist_file2consense (tot_pos_list, NULL, consense_file))
+	   {
+	     display_output_filename( stdout,"ConsenseTree","phylip",consense_file, CHECK);
+	   }
+	 else
+	   {
+	     fprintf ( stderr, "\nPhylip is not installed: the program could not produce the consense output. This is not mandatory but useful");
+	   }
+		  
+	 if ( (BT0=trmsdmat2tree (tdm, tcount, A)))
+	   {
+	     vfclose (print_tree (BT0,"newick", vfopen (struc_tree0, "w")));
+	     display_output_filename( stdout,"Tree","newick",struc_tree0, CHECK);
+	   }
+
 	 if ((BT10=treelist2filtered_bootstrap (T1, NULL,score, 0.1)))
 	   {
 	     vfclose (print_tree (BT10,"newick", vfopen (struc_tree10, "w")));
@@ -2390,6 +2414,122 @@ Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *results)
 	 exit (EXIT_SUCCESS);
 	 return NULL;
      }
+NT_node trmsdmat2tree (float **dm, int **count,Alignment *A)
+{
+  float min, max;
+  int s1, s2;
+  NT_node T;
+  int ns;
+  int **dm_int;
+  
+  ns=A->nseq;
+  for (s1=0; s1<ns-1; s1++)
+    for (s2=s1+1; s2<ns; s2++)
+      {
+	if ( count [s1][s2])dm[s1][s2]=dm[s2][s1]=dm[s1][s2]/(float)count[s1][s2];
+	else 
+	  {
+	    return NULL;
+	  }
+	if (s1==0 && s2==1)min=max=dm[s1][s2];
+	min=MIN(dm[s1][s2], min);
+	max=MAX(dm[s1][s2], max);
+      }
+  dm_int=declare_int (ns, ns);
+  for (s1=0; s1<A->nseq-1; s1++)
+    for (s2=s1+1; s2<A->nseq; s2++)
+      {
+	dm_int[s1][s2]=dm_int[s2][s1]=((dm[s1][s2])/(max))*100;
+      }
+  T=compute_std_tree_2(A, dm_int, "_TMODE_upgma");
+  free_int (dm_int, -1);
+  for (s1=0; s1<ns; s1++)for ( s2=0; s2<ns; s2++){dm[s1][s2]=count[s1][s2]=0;}
+  return T;
+}
+
+int pair_res_suitable4trmsd    (int s1,int col1, int col2, Alignment *A, int **pos, Pdb_param *PP, Constraint_list *CL,int *s)
+{
+   int rs;
+   rs=A->order[s1][0];
+   if ( !(CL->T[rs])){s[0]=s1; return 0;}
+   else if (is_gap (A->seq_al[s1][col1])){s[0]=s1;return 0;}
+   else if (is_gap (A->seq_al[s1][col2])){s[0]=s1;return 0;}
+   
+   else if (islower(A->seq_al[s1][col1])){s[0]=s1; return 0;}
+   else if (islower(A->seq_al[s1][col2])){s[0]=s1; return 0;}
+   
+   else if ( FABS(((pos[s1][col2])-(pos[s1][col1])))<=PP->n_excluded_nb){s[0]=s1;return 0;}
+   else if ((CL->T[rs])->ca_dist[pos[s1][col1]-1][pos[s1][col2]-1]==UNDEFINED){s[0]=s1;return 0;}
+   return 1;
+}
+int pair_columns_suitable4trmsd(int col1, int col2, Alignment *A, int **pos, Pdb_param *PP, Constraint_list *CL,int *s)
+{
+  int s1;
+  if (!column_is_suitable4trmsd (col1, A, pos, PP, CL,s))return 0;
+  if (!column_is_suitable4trmsd (col2, A, pos, PP, CL,s))return 0;
+  for (s1=0; s1<A->nseq; s1++)
+    {
+      int rs, rr1, rr2;
+      
+      rs=A->order[s1][0];
+      if ( FABS(((pos[s1][col2])-(pos[s1][col1])))<=PP->n_excluded_nb){s[0]=s1;return 0;}
+      if ((CL->T[rs])->ca_dist[pos[s1][col1]-1][pos[s1][col2]-1]==UNDEFINED){s[0]=s1;return 0;}
+      rr1=pos[s1][col1]-1;
+      rr2=pos[s1][col2]-1;
+      if ((CL->T[rs])->ca_dist[rr1][rr2]>PP->maximum_distance){s[0]=s1;return 0;}
+    }
+  return 1;
+}
+int column_is_suitable4trmsd(int col1,Alignment *A, int **pos, Pdb_param *PP, Constraint_list *CL,int *s)
+{
+  int s1;
+  for ( s1=0; s1<A->nseq; s1++)
+    {
+      int rs;
+      rs=A->order[s1][0];
+      if ( !(CL->T[rs])){s[0]=s1; return 0;}
+      else if (is_gap (A->seq_al[s1][col1])){s[0]=s1;return 0;}
+      else if (islower(A->seq_al[s1][col1])){s[0]=s1; return 0;}
+    }
+  return 1;
+}
+int aln_column_contains_gap (Alignment *A, int c)
+{
+  int a, b;
+  if ( !A || c>=A->len_aln || c<0)
+    {
+      printf ( "\nERROR: values out of range in aln_column_contains_gap [FATL:%s]\n", PROGRAM);
+      exit (EXIT_FAILURE);
+    }
+  for ( a=0; a<A->nseq; a++) if ( is_gap(A->seq_al[a][c]))return 1;
+  return 0;
+}
+  
+	   
+float aln2ncol4trmsd(Alignment *A, int **pos, Constraint_list *CL, int **lc)
+{
+  //This function estimates the number of columns suitable for constructing a trmsd
+  int col1, s1, ncol, n, rs1, real_res1_col1;
+  
+  for (s1=0; s1<A->nseq; s1++){lc[s1][0]=s1; lc[s1][1]=0;}
+  for (ncol=0,col1=0; col1< A->len_aln; col1++)
+    {
+      for (n=0,s1=0; s1<A->nseq; s1++)
+	{
+	  real_res1_col1=pos[s1][col1]-1;
+	  rs1=A->order[s1][0];
+		 
+	  if (real_res1_col1<0)lc[s1][1]++;
+	  else if (!((CL->T[A->order[s1][0]])->ca[real_res1_col1]))lc[s1][1]++;
+	  else n++;
+	}
+      if (n==A->nseq)
+	{
+	  ncol++;
+	}
+    }
+  return ncol;
+}
 
 float square_atom ( Atom *X)
 {
@@ -2647,7 +2787,7 @@ int display_rna_ss ( int n, char *seq, char *st, FILE *fp)
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Tue Oct 27 10:12:26 WEST 2009. */
+/*Tue Mar 30 19:55:28 WEDT 2010. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
