@@ -655,6 +655,23 @@ void get_rgb_values ( int val, Color *C)
 
     
      }
+
+// void output_tm_mark(FILE_format *fps)
+// {
+//    Color *box_c=vcalloc ( 1, sizeof (Color));
+//    Color *ink;
+//    get_rgb_values_format (INK_COLOR,     (ink  =vcalloc ( 1, sizeof (Color))));
+// 
+//    get_rgb_values_format ( 5, box_c);
+//    fps=print_format_char ( " IN ", box_c,ink,fps);
+// 
+//    get_rgb_values_format ( 9, box_c);
+//    fps=print_format_char ( " HEL ", box_c,ink,fps);
+// 
+//    get_rgb_values_format ( 0, box_c);
+//    fps=print_format_char ( "OUT", box_c,ink,fps);
+// }
+
 int output_color_format ( Alignment *B,Alignment *Sin,char *name, \
 FILE_format *(*vfopen_format)          ( char *),\
 FILE_format *(*print_format_string)    ( char * ,Color *, Color *, FILE_format*),\
@@ -736,6 +753,19 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
    sprintf (buf, "\n%s, %s(%s)\n%s\n",PROGRAM,VERSION,DATE, AUTHOR);     
    fps=print_format_string ( buf,white, ink, fps);
 
+   if(B->output_tm == 1)
+   {
+	fps=print_format_char ( '\n', white, ink, fps);
+	get_rgb_values_format ( 5, box_c);
+	fps=print_format_string ( " IN ", box_c,ink,fps);
+	
+	get_rgb_values_format ( 9, box_c);
+	fps=print_format_string ( " HEL ", box_c,ink,fps);
+	
+	get_rgb_values_format ( 0, box_c);
+	fps=print_format_string ( " OUT ", box_c,ink,fps);
+   }
+
    fps=print_format_string ( "\n\n",white,ink, fps);
    
    fps->line-=max_len;
@@ -1465,7 +1495,7 @@ int       output_seq_reliability_ascii     ( Alignment *B,Alignment *S, char *na
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Tue Oct 27 10:12:26 WEST 2009. */
+/*Tue Mar 30 19:55:28 WEDT 2010. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
