@@ -3,6 +3,7 @@
 #include <math.h>
 #include <stdarg.h>
 #include <string.h>
+#include <ctype.h>
 // #include <argp.h>
 
 #include "fast_tree_header.h"
@@ -86,7 +87,26 @@ tree_process_simple(NT_node tree,
 }
 
 
+void
+read_sequence_from_position(FILE *file, long position1, char *seq)
+{
+	fseek(file, position1, SEEK_SET);
+	char line[500];
+	int pos = -1;
+	while ((fgets(line, 500, file) != NULL) && (line[0] != '>'))
+	{
+		int l = -1;
+		while ((line[++l] != '\n') && (line[l] != '\0'))
+			seq[++pos] = line[l];
+	}
+	seq[++pos] = '\0';
+}
 
+// char*
+// read_sequence_from_position(FILE *file, long position1, long position2)
+// {
+// 	
+// }
 
 /**
 * Function to write tree to file in fastal_format. Leafs in \a tree do not need to be leafs in the complete tree.
@@ -214,38 +234,57 @@ void
 make_partTree(char *sequence_f,
 			  char *tree_f,
 			  int ktup,
-			  int subgroup)
+			  int subgroup,
+			  int is_dna,
+			  int retree)
 {
-	param_set = vcalloc(1,sizeof(PartTree_param));
-	param_set->ktup = ktup;
-	param_set->subgroup = subgroup;
-	
 	long *file_positions = NULL;
 	long **tmp1 = &file_positions;
 	int *seq_lengths = NULL;
 	int **tmp2 = &seq_lengths;
+	int number_of_sequences;
+	param_set = vcalloc(1,sizeof(PartTree_param));
+	param_set->ktup = ktup;
+	param_set->subgroup = subgroup;
+	Tree_fastal *tree;
 
-	//make index
-	int number_of_sequences =  make_pos_len_index_of_file(sequence_f, "KTUP_table", tmp1, tmp2, ktup, "DNA"); 
-	param_set->num_sequences = number_of_sequences;
-	param_set->ktup_positions = file_positions;
-	param_set->seq_lengths = seq_lengths;
-	param_set->threshold = 0.01;
-	param_set->ktup_table_f = fopen("KTUP_table","r");
 	
-	Tree_fastal *tree = vcalloc(number_of_sequences-1,sizeof(Tree_fastal));
-	param_set->tree = tree;
+	if (!retree)
+	{
+		//make index
+		char *ktup_table = vtmpnam(NULL);
+		param_set->num_sequences = number_of_sequences =  make_pos_len_index_of_file(sequence_f, ktup_table, tmp1, tmp2, ktup, is_dna);
+		tree = vcalloc(number_of_sequences-1,sizeof(Tree_fastal));
+		param_set->tree = tree;
+		param_set->ktup_positions = file_positions;
+		param_set->seq_lengths = seq_lengths;
+		param_set->threshold = 0.01;
+		param_set->ktup_table_f = fopen(ktup_table,"r");
 
-	int i;
-	partTree_r(param_set);
-// 	for (i = 0; i < number_of_sequences-1; ++i)
-// 	{
-// 		printf("%i %i %i\n", tree[i].left, tree[i].right, tree[i].name);
-// 	}
+	
+		partTree_r(param_set);
+	}
+	else
+	{
+
+		//make index
+		param_set->num_sequences = number_of_sequences =  make_pos_len_index_of_file_retree(sequence_f, tmp1, tmp2); 
+		tree = vcalloc(number_of_sequences-1,sizeof(Tree_fastal));
+		param_set->tree = tree;
+		param_set->ktup_positions = file_positions;
+		param_set->seq_lengths = seq_lengths;
+		param_set->threshold = 0.01;
+		param_set->ktup_table_f = fopen(sequence_f,"r");
+
+		partTree_retree(param_set);
+
+	}
+	
 	FILE * tree_file = fopen(tree_f,"w");
 	print_fastal_tree(tree, 0, tree_file, number_of_sequences);
 	fclose(tree_file);
 	vfree(tree);
+// 	exit(1);
 }
 
 
@@ -348,7 +387,6 @@ partTree_r(PartTree_param *param_set)
 	//get some memory
 	short *table1 = vcalloc(tsize, sizeof(short));
 	short *table2 = vcalloc(tsize, sizeof(short));
-	int *seed_set = vcalloc(param_set->subgroup, sizeof(int));
 	char **names = declare_char(param_set->subgroup, 8);
 	int **dist_mat = declare_int(param_set->subgroup, param_set->subgroup);
 	int **dist_mat2 = declare_int(param_set->subgroup, param_set->subgroup);
@@ -394,7 +432,6 @@ partTree_r(PartTree_param *param_set)
 		//if number of sequences in this group smaller than number subgoup size: make tree, finisch
 		if (num_sequences <= max_n_group)
 		{
-			int j;
 			dist_mat = make_distance_matrix(table_f, file_positions, index, num_sequences, dist_mat);
 			for (i = 0; i < num_sequences; ++i)
 			{
@@ -467,20 +504,7 @@ partTree_r(PartTree_param *param_set)
 		
 		
 		int num_in_clean = filter(index, dist_mat, seed_set_cleaned, param_set);
-// 		if (num_in_clean == 1)
-// 		{
-// 			int j;
-// // 			dist_mat = make_distance_matrix(table_f, file_positions, index, upgma_tree, dist_mat);
-// 			for (i = 0; i < param_set->subgroup; ++i)
-// 			{
-// 				sprintf(names[i],"%i", current_node->index[i]);
-// 			}
-// 			upgma_tree= (int_dist2upgma_tree_fastal (dist_mat, param_set->subgroup, file_name_tmp , names));
-// 			tree_process_simple(upgma_tree[0][0], param_set,loop_tree_node);
-// 			continue;
-// 		}
-// 		else
-// 		{
+
 		
 		if (num_in_clean ==1)
 		{
@@ -511,12 +535,7 @@ partTree_r(PartTree_param *param_set)
 			sprintf(names[i],"%i",i);
 		}
 		upgma_tree= (int_dist2upgma_tree_fastal (dist_mat2, num_in_clean, file_name_tmp , names));
-// 		}
-		
-// 		int *pos_tree_p = &param_set->pos_tree;
-
-		
-		int leaf = 0;
+ 
 
 		//cluster
 		//calculate distances from n' to N
@@ -632,6 +651,66 @@ make_distance_matrix(FILE *table_f,
 }
 
 
+int **
+make_distance_matrix_sim(FILE *aln_f,
+						 long *file_positions,
+						 int *sequence_group,
+						 int number,
+						 int **dist_mat)
+{
+	static char *seq1 = NULL;
+	static char *seq2;
+	char line[500];
+	if (seq1 == NULL)
+	{
+		int length = 0;
+		int i;
+		fseek(aln_f, file_positions[0], SEEK_SET);
+		fgets(line, 500, aln_f);
+		while (line[0] != '>')
+		{
+			i = -1;
+			while ((line[++i] != '\n') && (line[i] != '\0'));
+			length += i;
+			fgets(line, 500, aln_f);
+		}
+		seq1 = vcalloc(length+1, sizeof(short));
+		seq2 = vcalloc(length+1, sizeof(short));
+	}
+
+
+	int i, j, num = number-1, pos;
+	for (i = 0; i < num; ++i)
+	{
+
+		fseek(aln_f, file_positions[sequence_group[i]], SEEK_SET);
+  		
+		
+		pos = -1;
+		while ((fgets(line, 500, aln_f) != NULL) && (line[0] != '>'))
+		{
+			int l = -1;
+			while ((line[++l] != '\n') && (line[l] != '\0'))
+				seq1[++pos] = line[l];
+		}
+		seq1[++pos] = '\0';
+
+		for (j = i+1; j < number; ++j)
+		{
+			pos = -1;
+			fseek(aln_f, file_positions[sequence_group[j]], SEEK_SET);
+			while ((fgets(line, 500, aln_f) != NULL) && (line[0] != '>'))
+			{
+				int l = -1;
+				while ((line[++l] != '\n') && (line[l] != '\0'))
+					seq2[++pos] = line[l];
+			}
+			seq2[++pos] = '\0';
+			dist_mat[i][j] = dist_mat[j][i] = 100 - fast_aln2sim_mat2(seq1, seq2);
+		}
+	}
+	return dist_mat;
+}
 
 
 /**
@@ -691,7 +770,7 @@ makepointtable_fast(int *coded_seq,	//sequence
 * \param pointt Array with all ktups listed one after another. Has to end with END_ARRAY.
 * \param length length of \a table
 */
-int
+void
 makecompositiontable_fastal(FILE* tables_f,	//File to save the tables in
 							int *table,		//table to calculate in
 							int *pointt,	//ktups array
@@ -719,7 +798,7 @@ make_fast_tree(char *file_name,
 			   int ktup)
 {
 	
-	make_partTree(file_name, "TREE_OUT", ktup, n);
+	make_partTree(file_name, "TREE_OUT", ktup, n, 1, 0);
 
 }
 
@@ -877,15 +956,15 @@ make_pos_len_index_of_file(char *file_name,			//file with sequences
 						   long **file_positions,	//array to save the positions
 						   int **seq_lengths,		//array to save the sequence length
 						   int ktup,				//length of ktup
-						   char *type)				//type of the seuqence	
+						   int is_dna)				//type of the seuqence	
 {
 	//preparations for recoding sequence
 	int *aa;
-	int a, b, l;
+	int a, b;
 	
 	int ng = 0;
 	char **gl;
-	if ( strm (type, "DNA") || strm (type, "RNA"))
+	if (is_dna)
 	{
 		gl=declare_char (5,13);
 		sprintf ( gl[ng++], "Aa");
@@ -923,19 +1002,16 @@ make_pos_len_index_of_file(char *file_name,			//file with sequences
 			
 	(*file_positions) = vcalloc(ENLARGEMENT_PER_STEP,  sizeof(long));
 	(*seq_lengths) = vcalloc(ENLARGEMENT_PER_STEP,  sizeof(int));
-	int current_size = ENLARGEMENT_PER_STEP;
-	int current_pos = 0;
+
 
 	FILE *file = fopen(file_name,"r");
 
-
-	int seq_length=0;
 	char line[LINE_LENGTH];
 
 	int num_of_sequences = 0;
 	int str_len = 0;
 	int mem_for_pos = ENLARGEMENT_PER_STEP;
-	int tmp;
+
 	int real_len;
 	int *c_seq;
 
@@ -959,7 +1035,7 @@ make_pos_len_index_of_file(char *file_name,			//file with sequences
 			}
 			
 			int i;
-			int length = strlen(line);
+			
 			if (line[0] == '>')
 			{
 				if (num_of_sequences >0)
@@ -994,7 +1070,7 @@ make_pos_len_index_of_file(char *file_name,			//file with sequences
 					--real_len;
 				for (i = 0; i < real_len; ++i)
 				{
-					coded_seq[str_len++] = aa[line[i]];
+					coded_seq[str_len++] = aa[(short)line[i]];
 				}
 			}
 		}
@@ -1009,11 +1085,390 @@ make_pos_len_index_of_file(char *file_name,			//file with sequences
 	fclose(tables_f);
 	return num_of_sequences;
 }
+
+
+/**
+ *	Makes an index of a file
+ */
+int
+make_pos_len_index_of_file_retree(char *file_name,			//file with sequences
+								  long **file_positions,	//array to save the positions
+								  int **seq_lengths)		//array to save the sequence length
+{
+
+// 	printf("HALLO\n");
+	//Reading sequences
+	const int LINE_LENGTH = 501;
+	(*file_positions) = vcalloc(ENLARGEMENT_PER_STEP,  sizeof(long));
+	(*seq_lengths) = vcalloc(ENLARGEMENT_PER_STEP,  sizeof(int));
+
+
+	FILE *file = fopen(file_name,"r");
+
+	char line[LINE_LENGTH];
+
+	int num_of_sequences = 0;
+	int mem_for_pos = ENLARGEMENT_PER_STEP;
+// 	fgets(line, LINE_LENGTH , file)
+	 int seq_len = 0;
+			
+			
+	if (file == NULL)
+	{
+		printf("FILE NOT FOUND\n");
+		exit(1);
+	}
+	else
+	{
+		int i;
+		while(fgets(line, LINE_LENGTH , file)!=NULL)
+		{
+// 			line[LINE_LENGTH -2] = '\n';
+			if (line[0] == '>')
+			{
+				(*file_positions)[num_of_sequences] = ftell(file);
+				if (num_of_sequences >0)
+				{
+					(*seq_lengths)[num_of_sequences-1] = seq_len;
+					seq_len = 0;
+				}
+				++num_of_sequences;
+
+				if (num_of_sequences == mem_for_pos)
+				{
+					mem_for_pos += ENLARGEMENT_PER_STEP;
+					(*file_positions) = vrealloc((*file_positions), mem_for_pos * sizeof(long));
+					(*seq_lengths) = vrealloc((*seq_lengths), mem_for_pos * sizeof(int));
+				}
+			}
+			else
+			{
+				i = -1;
+				while ((line[++i] != '\n') && (line[i] != '\0'))
+				{
+					if (line[i] != '-')
+					{
+// 						printf("A: %c\n", line[i]);
+						++seq_len;
+					}
+				}
+			}
+		}
+	}
+	(*seq_lengths)[num_of_sequences-1] = seq_len;
+// 	printf("%i %li\n", (*seq_lengths)[0], (*file_positions)[0]);
+// 	printf("%i %li\n", (*seq_lengths)[1], (*file_positions)[1]);
+	fclose(file);
+	return num_of_sequences;
+}
+
+
+
+int logid_score2 ( int sim, int len)
+{
+	float score;
+  
+	if ( len==0)return (int)(0.33*(float)MAXID);
+  
+	score=(float)sim/(float)len;
+	if (score>0.9) score=1.0;
+	else score=-log10 (1.0-score);
+  
+	score=(score*MAXID);
+	return score;
+}
+
+
+int fast_aln2sim_mat2 (char *seq1, char *seq2)
+{
+	int r1, r2;
+	int len = 0;
+	int sim = 0;
+	int c = -1;
+	int simm=100;
+// 	printf("SEQ: %s %s\n", seq1, seq2);
+	while (seq1[++c] != '\0')
+	{
+		r1=tolower (seq1[c]);
+		r2=tolower (seq2[c]);
+		if ((r1 == '-') && (r2 == '-')) continue;
+		if (r1==r2) sim++;
+		len++;
+	}
+
+
+	simm = logid_score2 ( sim, len);
+	return simm;
+}
+
+
+
+/*!
+ *	\brief Function to create a tree using the PartTree algorithm.
+ *	
+ *	\param param_set A \a PartTree_param object containing all necessary parameters and the data.
+ *	\return The node_number.
+ */
+void
+partTree_retree(PartTree_param *param_set)
+{
+	int num_of_tree_nodes = param_set->num_sequences-1;
+	int loop_tree_node;
+
+	Tree_fastal *tree = param_set->tree;
+// 	int this_node = param_set->pos_tree;
+	
+	int i;
+// 	int tsize = param_set->tsize;
+	
+	
+	//get some memory
+// 	short *table1 = vcalloc(tsize, sizeof(short));
+// 	short *table2 = vcalloc(tsize, sizeof(short));
+	char **names = declare_char(param_set->subgroup, 8);
+	int **dist_mat = declare_int(param_set->subgroup, param_set->subgroup);
+	int **dist_mat2 = declare_int(param_set->subgroup, param_set->subgroup);
+	char * file_name_tmp = vtmpnam(NULL);
+	int *seed_set_cleaned = vcalloc(param_set->subgroup, sizeof(int));
+	FILE *aln_f = param_set->ktup_table_f;
+	long *file_positions = param_set->ktup_positions;
+	int max_n_group = param_set->subgroup;
+	int num_in_subgroup = param_set->subgroup;
+	int *seq_lengths = param_set->seq_lengths;
+	int *clusters = vcalloc(param_set->subgroup+1, sizeof(int));
+	int *min_dist = vcalloc(param_set->num_sequences, sizeof(int));
+	int *belongs_to = vcalloc(param_set->num_sequences, sizeof(int));
+
+	int aln_length = 1;
+	fseek(aln_f, file_positions[0], SEEK_SET);
+	char line[500];
+	while ((fgets(line, 500, aln_f) != NULL) && (line[0] != '>'))
+	{
+		i = -1;
+		while ((line[++i] != '\n') && (line[i] != '\0'));
+		aln_length += i;
+	}
+	char *seq1 = vcalloc(aln_length, sizeof(char));
+	char *seq2 = vcalloc(aln_length, sizeof(char));
+	
+	//Prepare first node
+	
+	tree[0].index = vcalloc(param_set->num_sequences,sizeof(int));
+	int *index = tree[0].index;
+	for (i = 0; i< param_set->num_sequences; ++i)
+		index[i] = i;
+	tree[0].name = param_set->pos_tree +param_set->num_sequences;
+	
+	tree[0].num_leafs = param_set->num_sequences;
+	int *sequence_group2 = vcalloc(param_set->num_sequences,sizeof(int));
+// 	
+	Tree_fastal *current_node;
+	for (loop_tree_node = 0; loop_tree_node < num_of_tree_nodes; ++loop_tree_node)
+	{
+// 		printf("ROUND: %i\n", loop_tree_node);
+		current_node = &tree[loop_tree_node];
+		index= current_node->index;
+		if (current_node->index == NULL)
+		{
+			continue;
+		}
+		int num_sequences = current_node->num_leafs;
+
+		//if number of sequences in this group smaller than number subgoup size: make tree, finisch
+		if (num_sequences <= max_n_group)
+		{
+			dist_mat = make_distance_matrix_sim(aln_f, file_positions, index, num_sequences, dist_mat);
+			for (i = 0; i < num_sequences; ++i)
+			{
+				sprintf(names[i],"%i", current_node->index[i]);
+	 		}
+			NT_node **tree= (int_dist2upgma_tree_fastal (dist_mat, num_sequences, file_name_tmp , names));
+			tree_process_simple(tree[0][0], param_set,loop_tree_node);
+			continue;
+		}
+
+
+		for (i = 0; i < num_in_subgroup; ++i)
+		{
+			seed_set_cleaned[i] = 1;
+		}
+		
+		//finde longest sequence and put into the first field
+		
+		int index_longest = 0;
+		int length_of_longest = 0;
+		
+		for(i = 0; i < num_sequences; ++i)
+		{
+			if (seq_lengths[index[i]] > length_of_longest)
+			{
+				index_longest = i;
+				length_of_longest = seq_lengths[index[i]];
+			}
+		}
+		int tmp = index[index_longest];
+		index[index_longest] = index[0];
+		index[0] = tmp;
+
+		//distance of longest to rest
+		int seq_index = 1;
+		read_sequence_from_position(aln_f, file_positions[index[0]], seq1);
+		int min = -1;
+		
+		
+		for (i = 1; i < num_sequences; ++i)
+		{
+			read_sequence_from_position(aln_f, file_positions[index[1]], seq2);
+			tmp = 100 - fast_aln2sim_mat2(seq1, seq2);	
+			if (tmp < min)
+			{
+				min = tmp;
+				seq_index = i;
+			}
+		}
+
+		//get the new seed_set in the first n spaces
+		tmp = index[1];
+		index[1] = index[seq_index];
+		index[seq_index] = tmp;
+		int r,j;
+		num_in_subgroup = param_set->subgroup;
+
+		
+		for (i = 2; i < num_in_subgroup; ++i)
+		{
+			r = i + rand() / ( RAND_MAX / ( num_sequences-i) + 1 );
+			tmp = index[r];
+			index[r] = index[i];
+			index[i] = tmp;
+		}
+
+		//Calculate matrix
+		dist_mat = make_distance_matrix_sim(aln_f, file_positions, index, param_set->subgroup, dist_mat);
+// 	
+// 		//Filter out sequences that are to similar & reorder
+// 		
+		NT_node **upgma_tree;
+		
+		
+		int num_in_clean = filter(index, dist_mat, seed_set_cleaned, param_set);
+
+		
+		if (num_in_clean ==1)
+		{
+			num_in_clean = 2;
+			seed_set_cleaned[1] = 1;
+		}
+			//make_tree
+		int col = 0;
+		int row = 0;
+		for (i = 0; i <  num_in_subgroup; ++i)
+		{
+			if (seed_set_cleaned[i])
+			{
+				row = col+1;
+				for (j = i+1; j <  num_in_subgroup; ++j)
+				{
+					if (seed_set_cleaned[j])
+					{
+						dist_mat2[row][col] = dist_mat2[col][row] = dist_mat[i][j];
+						++row;
+					}
+				}
+				++col;
+			}
+		}
+		for (i = 0; i < num_in_clean; ++i)
+		{
+			sprintf(names[i],"%i",i);
+		}
+		upgma_tree= (int_dist2upgma_tree_fastal (dist_mat2, num_in_clean, file_name_tmp , names));
+ 
+
+// 		//cluster
+// 		//calculate distances from n' to N
+		read_sequence_from_position(aln_f, file_positions[index[0]], seq1);
+// 		get_table(table1, table_f, file_positions[index[0]]);
+		for (j = num_in_clean; j < num_sequences; ++j)
+		{
+			read_sequence_from_position(aln_f, file_positions[index[j]], seq2);
+			min_dist[j] = 100 - fast_aln2sim_mat2(seq1, seq2);	
+// 			min_dist[j] = euclidean_dist_half(table_f, file_positions[index[j]], table1, table2, param_set->tsize);
+			belongs_to[j] = 0;
+		}
+		for(i = 1; i < num_in_clean; ++i)
+		{
+			read_sequence_from_position(aln_f, file_positions[index[0]], seq1);
+// 			get_table(table1, table_f, file_positions[index[i]]);
+			belongs_to[i] = i;
+			for (j = num_in_clean; j < num_sequences; ++j)
+			{
+				read_sequence_from_position(aln_f, file_positions[index[j]], seq2);
+				tmp = 100 - fast_aln2sim_mat2(seq1, seq2);
+// 				tmp = euclidean_dist_half(table_f, file_positions[index[j]], table1, table2, param_set->tsize);
+				if (tmp < min_dist[j])
+				{
+					min_dist[j] = tmp;
+					belongs_to[j] = i;
+				}
+			}
+		}
+// 
+		//how_many sequences has each cluster
+		for (j = 0; j <= num_in_subgroup; ++j)
+		{
+			clusters[j] = 0;
+		}
+		for (j = 0; j < num_sequences; ++j)
+		{
+			++clusters[belongs_to[j]];
+		}
+// 		for (j = 0; j <= num_in_subgroup; ++j)
+// 		{
+// 			printf("CL: %i ",clusters[j]);
+// 		}
+// 		printf("\n");
+		for(i = 1; i < num_in_clean; ++i)
+		{
+			clusters[i] += clusters[i-1];
+		}
+		clusters[num_in_clean] = clusters[num_in_clean-1];
+
+		for (i = 0; i < num_sequences; ++i)
+		{
+			sequence_group2[--clusters[belongs_to[i]]] = index[i];
+		}
+
+		for (i = 0; i < num_sequences; ++i)
+		{
+			index[i] = sequence_group2[i];
+		}
+
+		
+		for (i = 0; i < num_in_clean; ++i)
+		{
+			sprintf(names[i],"%i",i);
+		}
+		tree_process(upgma_tree[0][0], param_set, clusters, index, loop_tree_node);
+		NT_node tmp_tree = upgma_tree[3][0];
+		vfree(upgma_tree[0]);
+		vfree(upgma_tree[1]);
+		vfree(upgma_tree[2]);
+		vfree(upgma_tree[3]);
+		vfree(upgma_tree);
+		free_tree(tmp_tree);
+	}
+// 	exit(1);
+	vfree(min_dist);
+	vfree(belongs_to);
+	vfree(clusters);
+}
+
 /*********************************COPYRIGHT NOTICE**********************************/
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Tue Oct 27 10:12:26 WEST 2009. */
+/*Tue Mar 30 19:55:28 WEDT 2010. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
