@@ -234,7 +234,9 @@ Constraint_list *copy_constraint_list (Constraint_list *CL, int mode)
     /*log*/
       sprintf ( NCL->method_log, "%s",CL->method_log);
       sprintf ( NCL->evaluate_mode, "%s",CL->evaluate_mode);
-      
+    /* Gene Prediction*/
+      sprintf ( NCL->genepred_score, "%s",CL->genepred_score);
+
     /*Parameters for domain extraction*/
       NCL->moca=(mode==HARD_COPY)?duplicate_moca ( CL->moca):CL->moca;
       
@@ -523,7 +525,23 @@ Structure *extend_structure ( Structure *S)
     S->n_fields++;
     return S;
     }
-
+Sequence * reset_sequence_len (Sequence *S)
+{
+  int min,max,a,l;
+ 
+  if ( !S || !S->nseq)return 0;
+  max=min=strlen (S->seq[0]);
+  for (a=0; a<S->nseq; a++)
+    {
+      l=strlen (S->seq[a]);
+      min=MIN(l,min);
+      max=MAX(l,max);
+      S->len[a]=l;
+    }
+  S->max_len=max;
+  S->min_len=min;
+  return S;
+}
 Sequence * declare_sequence ( int min, int max, int nseq)
     {
     Sequence *LS;
@@ -776,7 +794,7 @@ Alignment* copy_aln ( Alignment *A, Alignment *B)
 
 	    if ( (A->S)==NULL){vfree (B->score_seq); B->score_seq=vcalloc ( A->max_n_seq, sizeof (int));}
 	    ga_memcpy_int(  A->score_seq,B->score_seq,B->nseq);
-	    
+	    B->score_res=A->score_res;
 
 	    B->score_aln=A->score_aln;
 	    B->score=A->score;
@@ -1064,6 +1082,21 @@ long aln_stack (Alignment *A, int mode)
 }
 
 Sequence* free_aln ( Alignment *LA){ return free_Alignment(LA);}
+Alignment *free_data_in_aln (Alignment *A)
+{
+  //Frees only the sequence data (keeps profile information)
+  free_char (A->seq_al, -1);
+  free_char (A->seq_comment, -1);
+  free_char (A->aln_comment, -1);
+  free_char (A->name, -1);
+  free_char (A->expanded_order, -1);
+  free_char (A->seq_al, -1);
+  free_int (A->order, -1);
+  free_int (A->seq_cache, -1);
+  free_int (A->cdna_cache, -1);
+  free_int (A->score_res, -1);
+  free_sequence (A->S, -1);
+}
 Sequence* free_Alignment ( Alignment *LA)
 	{
 	  /* Does not free the A->S field (sequences of A)*/
@@ -1805,7 +1838,7 @@ Fname *free_fname ( Fname *F)
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Tue Oct 27 10:12:26 WEST 2009. */
+/*Tue Mar 30 19:55:28 WEDT 2010. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
