@@ -911,7 +911,7 @@ int get_tot_prob (Alignment *A1,Alignment *A2, int *ns, int **ls, int nstates, f
 	{
 	  int in;
 	  c1=tolower(A1->seq_al[ls[0][b]][i]);
-	  if ( c1=='-')continue;
+	  if ( c1=='-' || c1=='.' || c1=='~')continue;
 	  c1-='a';
 	  
 	  if (!(in=observed[c1])){in=observed[c1]=++index;}
@@ -1628,7 +1628,7 @@ float ** get_emitPairs (char *mat, char *alp, float **p, float *s)
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Tue Oct 27 10:12:26 WEST 2009. */
+/*Tue Mar 30 19:55:28 WEDT 2010. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
