@@ -230,8 +230,7 @@ Job_TC*** split_job_list (Job_TC *job, int ns)
  
   if   (nj==0)return NULL;
   else split=(nj/ns)+1;
-  
-
+   
   n=a=u=0;
   jl[a][0]=job;
   while (job)
@@ -252,6 +251,30 @@ Job_TC*** split_job_list (Job_TC *job, int ns)
       n++;
       job=job->c;
     }
+
+  /*Display job details when debugging
+  a=0;
+  while (jl[a])
+    {
+      Job_TC *start,* end;
+      int todo=0;
+      
+      start=job=jl[a][0];
+      end=jl[a][1];
+      while (job!=end){todo++;job=job->c;}
+      job=start;
+      while (job!=end)
+	{
+	  HERE ("--- %d %d %s",a, job, job->param->aln_c);
+	  job=job->c;
+	}
+      a++;
+    }
+  
+    a=0;
+    while (jl[a]){HERE ("**** %d %d ", jl [a][0], jl[a][1]);a++;}
+    exit (0);
+  */
   return jl;
 }
 
@@ -292,7 +315,7 @@ Job_TC* retrieve_job ( Job_TC *job)
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Tue Oct 27 10:12:26 WEST 2009. */
+/*Tue Mar 30 19:55:28 WEDT 2010. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
