@@ -1289,7 +1289,7 @@ NT_node upgma_merge (int **mat, NT_node *NL, int *used, int *n, int N)
 //
 //                              SPLIT UPGMA
 ///////////////////////////////////////////////////////////////////////////////
-int upgma_node_heap (NT_node X);
+
 int upgma_node_heap (NT_node X)
 {
   static int n;
@@ -1445,7 +1445,7 @@ float get_split_dist ( Alignment *A, NT_node L, NT_node R, Split **S)
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Tue Oct 27 10:12:26 WEST 2009. */
+/*Tue Mar 30 19:55:28 WEDT 2010. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
